/*
 * Test for memory leak when creating and destroying context and integrator
 *
 * Joshua L. Adelman
 * jla65@pitt.edu
 */

#include "OpenMM.h"

#include <cstdio>
#include <stdio.h>
#include <math.h>
#include <list>
#include <iostream>
#include <time.h>

using namespace OpenMM;
using namespace std;

/*******************************************************************
 * MAIN
 *******************************************************************/

int main() {
    // Load OpenMM plugins (allowing for CUDA platform).
    Platform::loadPluginsFromDirectory(Platform::getDefaultPluginsDirectory());
    
    int numberofblocks = 2000;
    int stepsperblock = 100;
    Platform& platform         =  Platform::getPlatformByName("OpenCL");
    platform.setPropertyDefaultValue("OpenCLDeviceIndex","0");
    //printf( "Using OpenMM platform %s\n", context->getPlatform().getName().c_str() );

    double temperature = 298.0; // thermostat bath temperature, K
    double collision_frequency = 90.0; // thermostat collision frequency, 1/ps
    double timestep = 0.001; // molecular dyanamics timestep, ps

    // Default parameters
    double mass_Ar     = 39.9; // amu
    double q_Ar        = 0.0; // elementary_charge
    double sigma_Ar    = 0.3350; // nm
    double epsilon_Ar  = 0.001603; // kilojoule_per_mole

    // Set spring constant for centrail restraint.
    double K = 1.0; // units.kilojoules_per_mole/units.nanometer**2

    // Grid dimensions for initial lattice.
    int nx = 10;
    int ny = 10;
    int nz = 10;
    
    double scaleStepSizeX = 1.0;
    double scaleStepSizeY = 1.0;
    double scaleStepSizeZ = 1.0;

    // Determine total number of atoms.
    int natoms = nx * ny * nz;

    // Create an empty system object.
    System system;

    // Create a NonbondedForce object with no cutoff.
    NonbondedForce* nb = new NonbondedForce();
    nb->setNonbondedMethod(NonbondedForce::NoCutoff);

    // Allocate coordinate storage.
    vector<Vec3> coordinates(natoms);
    
    int atom_index = 0;
    for(int ii = 0; ii < nx; ii++)
      for(int jj = 0; jj < ny; jj++) 
	for(int kk = 0; kk < nz; kk++) {
	  system.addParticle(mass_Ar);
	  nb->addParticle(q_Ar, sigma_Ar, epsilon_Ar);
	  double x = sigma_Ar*scaleStepSizeX*(float(ii) - nx/2.0);
	  double y = sigma_Ar*scaleStepSizeY*(float(jj) - ny/2.0);
	  double z = sigma_Ar*scaleStepSizeZ*(float(kk) - nz/2.0);

	  coordinates[atom_index] = Vec3(x,y,z);
	  atom_index++;
	}

    // Add the nonbonded force.
    system.addForce(nb);

	// Define a Langevin integrator.
	LangevinIntegrator * integrator = new LangevinIntegrator(temperature, collision_frequency, timestep);

	// Initialize Context
    Context * context = new Context(system, *integrator, platform);
    
    /*
     * Loop over blocks creating and destroying integrator and context each time
     */
	for(int ii = 0; ii < numberofblocks; ii++) {
    	
    	// Set positions.
    	context->setPositions(coordinates);
    	
    	// Run some dynamics
    	integrator->step(stepsperblock);

		// Compute forces from reference context.
		State state = context->getState(State::Positions+State::Energy);
		coordinates = state.getPositions();  
		
		printf("step %d of %d: Potential energy %f\n",ii,numberofblocks,state.getPotentialEnergy());
	
		
		
	}

   	// Delete context.
	delete context;
	
	// Delete integrator.
	delete integrator;

    return 0;
}
